@@ -1,6 +1,6 @@
 /* The MIT License
 
-   Copyright (c) 2023 Genome Research Ltd.
+   Copyright (c) 2023-2025 Genome Research Ltd.
 
    Author: Petr Danecek <pd3@sanger.ac.uk>
 
@@ -36,7 +36,7 @@
 
     Read about transcript types here
         http://vega.sanger.ac.uk/info/about/gene_and_transcript_types.html
-        http://www.ensembl.org/info/genome/variation/predicted_data.html
+        https://www.ensembl.org/info/genome/variation/prediction/predicted_data.html
         https://www.gencodegenes.org/pages/biotypes.html
 
     List of supported biotypes
@@ -289,7 +289,6 @@ typedef enum
 {
     // write options
     verbosity,          // int, 0-2
-    strip_chr_names,    // int, 0 to leave as is, 1 to strip 'chr' prefix
     force_out_of_phase, // int, 1 to proceed even CDS exon out of expected phase
     dump_fname,         // const char*, dump the parsed GFF into this file, for debugging purposes
 
@@ -314,4 +313,9 @@ void *gff_get(gff_t *gff, gff_opt_t key);
 const char *gff_id2string(gff_t *gff, id_type_t type, int id);
 const char *gf_type2gff_string(int type);
 
+int gff_has_seq(gff_t *gff, const char *chr);
+int gff_nseq(gff_t *gff);
+const char *gff_iseq(gff_t *gff, int i);
+
+
 #endif
