@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2016-2023 Genome Research Ltd.
+    Copyright (C) 2016-2025 Genome Research Ltd.
 
     Author: Petr Danecek <pd3@sanger.ac.uk>
 
@@ -61,7 +61,7 @@ static const char *usage_text(void)
         "\n"
         "About: Compare two files and set non-identical genotypes in the first file to missing.\n"
         "\n"
-        "Usage: bcftools +isecGT <A.bcf> <B.bcf> [Plugin Options]\n"
+        "Usage: bcftools +isecGT A.bcf B.bcf [Plugin Options]\n"
         "Plugin options:\n"
         "   -o, --output FILE               Write output to a file [standard output]\n"
         "   -O, --output-type u|b|v|z[0-9]  u/b: un/compressed BCF, v/z: un/compressed VCF, 0-9: compression level [v]\n"
@@ -69,7 +69,8 @@ static const char *usage_text(void)
         "   -R, --regions-file FILE         Restrict to regions listed in a file\n"
         "   -t, --targets REGION            Similar to -r but streams rather than index-jumps\n"
         "   -T, --targets-file FILE         Similar to -R but streams rather than index-jumps\n"
-        "       --write-index               Automatically index the output files [off]\n"
+        "   -v, --verbosity INT             Verbosity level\n"
+        "   -W, --write-index[=FMT]         Automatically index the output files [off]\n"
         "\n";
 }
 
@@ -87,15 +88,19 @@ int run(int argc, char **argv)
         {"targets-file",required_argument,NULL,'T'},
         {"output",required_argument,NULL,'o'},
         {"output-type",required_argument,NULL,'O'},
-        {"write-index",no_argument,NULL,1},
+        {"write-index",optional_argument,NULL,'W'},
+        {"verbosity",required_argument,NULL,'v'},
         {NULL,0,NULL,0}
     };
     int c;
     char *tmp;
-    while ((c = getopt_long(argc, argv, "o:O:r:R:t:T:",loptions,NULL)) >= 0)
+    while ((c = getopt_long(argc, argv, "o:O:r:R:t:T:W::v:",loptions,NULL)) >= 0)
     {
         switch (c)
         {
+            case 'v':
+                if ( apply_verbosity(optarg) < 0 ) error("Could not parse argument: --verbosity %s\n", optarg);
+                break;
             case 'o': args->output_fname = optarg; break;
             case 'O':
                       switch (optarg[0]) {
@@ -119,7 +124,10 @@ int run(int argc, char **argv)
             case 'R': args->regions_list = optarg; args->regions_is_file = 1; break;
             case 't': args->targets_list = optarg; break;
             case 'T': args->targets_list = optarg; args->targets_is_file = 1; break;
-            case  1 : args->write_index = 1; break;
+            case 'W':
+                if (!(args->write_index = write_index_parse(optarg)))
+                    error("Unsupported index format '%s'\n", optarg);
+                break;
             case 'h':
             case '?':
             default: error("%s", usage_text()); break;
@@ -151,7 +159,9 @@ int run(int argc, char **argv)
     args->out_fh = hts_open(args->output_fname ? args->output_fname : "-", wmode);
     if ( args->out_fh == NULL ) error("Can't write to \"%s\": %s\n", args->output_fname, strerror(errno));
     if ( bcf_hdr_write(args->out_fh, args->hdr_a)!=0 ) error("[%s] Error: cannot write to %s\n", __func__,args->output_fname);
-    if ( args->write_index && init_index(args->out_fh,args->hdr_a,args->output_fname,&args->index_fn)<0 ) error("Error: failed to initialise index for %s\n",args->output_fname);
+    if ( init_index2(args->out_fh,args->hdr_a,args->output_fname,
+                     &args->index_fn, args->write_index)<0 )
+      error("Error: failed to initialise index for %s\n",args->output_fname);
 
     while ( bcf_sr_next_line(args->sr) )
     {
