@@ -1,6 +1,6 @@
 /* The MIT License
 
-   Copyright (c) 2018-2024 Genome Research Ltd.
+   Copyright (c) 2018-2025 Genome Research Ltd.
 
    Author: Petr Danecek <pd3@sanger.ac.uk>
 
@@ -153,6 +153,8 @@ static const char *usage_text(void)
         "   -T, --targets-file FILE         Similar to -R but streams rather than index-jumps\n"
         "       --targets-overlap 0|1|2     Include if POS in the region (0), record overlaps (1), variant overlaps (2) [0]\n"
         "       --no-version                Do not append version and command line to the header\n"
+        "   -v, --verbosity INT             Verbosity level\n"
+        "   -W, --write-index[=FMT]         Automatically index the output files [off]\n"
         "\n"
         "General options:\n"
         "   -m, --min-score NUM             Do not add FMT/DNM annotation if the score is smaller than NUM\n"
@@ -181,7 +183,6 @@ static const char *usage_text(void)
         "       --use-NAIVE                 A naive calling model which uses only FMT/GT to determine DNMs\n"
         "       --with-pAD                  Do not use FMT/QS but parental FMT/AD\n"
         "       --with-pPL                  Do not use FMT/QS but parental FMT/PL. Equals to DNG with bugs fixed (more FPs, fewer FNs)\n"
-        "   -W, --write-index[=FMT]         Automatically index the output files [off]\n"
         "\n"
         "Example:\n"
         "   # Annotate VCF with FORMAT/DNM, run for a single trio\n"
@@ -510,7 +511,7 @@ static void init_tprob_mprob(args_t *args, int fi, int mi, int ci, double *tprob
     *denovo_allele = ca!=fa && ca!=fb && ca!=ma && ca!=mb ? ca : cb;
 
     // tprob .. genotype transmission probability L(GC|GM,GF), 0 if not compatible with Mendelian inheritance
-    // mprob .. probability of mutation
+    // mprob .. probability of mutation, 0 if this case is forbidden and not considered at all
 
     int is_novel;
     if ( args->strictly_novel )
@@ -542,8 +543,10 @@ static void init_tprob_mprob(args_t *args, int fi, int mi, int ci, double *tprob
         else *mprob = args->mrate * args->mrate;
     }
 }
-static void init_tprob_mprob_chrX(args_t *args, int mi, int ci, double *tprob, double *mprob, int *denovo_allele)
+static void init_tprob_mprob_chrX(args_t *args, int fi, int mi, int ci, double *tprob, double *mprob, int *denovo_allele)
 {
+    int fa = seq1[fi];
+    int fb = seq2[fi];
     int ma = seq1[mi];
     int mb = seq2[mi];
     int ca = seq1[ci];
@@ -551,8 +554,17 @@ static void init_tprob_mprob_chrX(args_t *args, int mi, int ci, double *tprob, d
 
     *denovo_allele = ca!=ma && ca!=mb ? ca : cb;
 
-    if ( ca!=cb )                   // male cannot be heterozygous in X
-        *mprob = 0, *tprob = 0;
+    // tprob .. genotype transmission probability L(GC|GM,GF), 0 if not compatible with Mendelian inheritance
+    // mprob .. probability of mutation, 0 if this case is forbidden and not considered at all
+
+    if ( ca!=cb )   // male cannot be heterozygous in X, but it can be mosaic
+    {
+        int is_novel = ( (ca!=fa && ca!=fb && ca!=ma && ca!=mb) || (cb!=fa && cb!=fb && cb!=ma && cb!=mb) ) ? 1 : 0;
+        if ( is_novel )
+            *mprob = args->mrate, *tprob = 0;
+        else
+            *mprob = 0, *tprob = 0;
+    }
     else if ( ca==ma || ca==mb )    // inherited
     {
         if ( ma==mb ) *tprob = 1;
@@ -573,6 +585,9 @@ static void init_tprob_mprob_chrXX(args_t *args, int fi, int mi, int ci, double
 
     *denovo_allele = ca!=fa && ca!=fb && ca!=ma && ca!=mb ? ca : cb;
 
+    // tprob .. genotype transmission probability L(GC|GM,GF), 0 if not compatible with Mendelian inheritance
+    // mprob .. probability of mutation, 0 if this case is forbidden and not considered at all
+
     if ( fa!=fb )
     {
         // this must be a genotype error, father cannot be heterozygous in X; don't flag it as a DNM unless
@@ -623,7 +638,7 @@ static void init_priors(args_t *args, priors_t *priors, init_priors_t type)
                 else if ( type==autosomal )
                     init_tprob_mprob(args,fi,mi,ci,&tprob,&mprob,&allele);
                 else if ( type==chrX )
-                    init_tprob_mprob_chrX(args,mi,ci,&tprob,&mprob,&allele);
+                    init_tprob_mprob_chrX(args,fi,mi,ci,&tprob,&mprob,&allele);
                 else if ( type==chrXX )
                     init_tprob_mprob_chrXX(args,fi,mi,ci,&tprob,&mprob,&allele);
                 else
@@ -958,8 +973,17 @@ static double process_trio_ACM(args_t *args, priors_t *priors, int nals, double
     {
         // Downplay de novo calls with alleles present in the parents
         int ial = *al1;
-        sum = sum_log(sum,qs[iMOTHER][ial] + qs[iFATHER][ial]);
-        max += qs[iMOTHER][ial] + qs[iFATHER][ial];
+        if ( qs[iMOTHER][ial] + qs[iFATHER][ial] != 0 )
+        {
+            double tmp = 0;
+            if ( qs[iMOTHER][ial] ) tmp += subtract_log(0,qs[iMOTHER][ial]);
+            if ( qs[iFATHER][ial] ) tmp += subtract_log(0,qs[iFATHER][ial]);
+            sum = sum_log(sum,tmp);
+            max += tmp;
+#if DEBUG
+            fprintf(stderr,"max=%e sum=%e   ret=%e  after adjusting with --strictly-novel\n",max,sum,max-sum);
+#endif
+        }
     }
 
     // This is the log( 1 - (\max L_pfm) / (\sum L_pfm) ). The default output (DNM:log) prints the inverse. Note log
@@ -1177,6 +1201,24 @@ static void set_trio_QS_noisy(args_t *args, trio_t *trio, double *pqs[3], int nq
                 if ( pns < pnoise->abs1 * sum_qs / sum_ad ) pns = pnoise->abs1 * sum_qs / sum_ad;
             }
         }
+        // Reduce QS for all alleles to account for noise. Note this has one caveat: low-VAF proband and
+        // parental sites (13% and 8%) leave the het genotype in the proband very likely (since PL is used in
+        // child, not QS), but shift parental genotype toward hom. For example, if AD_c=53,8 and AD_f=128,7
+        // the DNM score can be still DNM=-5.98923 (see chrX:141907033 in test/trio-dnm/trio-dnm.11.vcf)
+        for (k=0; k<nqs1; k++)
+        {
+            double val = qs[k];
+            if ( n_ad && (!ad_f[k] || !ad_m[k]) ) val -= pns;
+            else val -= pn;
+            if ( val < 0 ) val = 0;
+            pqs[j][k] = phred2log(val);
+        }
+    }
+#if 0
+        // This worked well with one caveat: low-VAF proband and parental sites (13% and 8%) leave
+        // the het genotype in the proband very likely (since PL is used in child, not QS), but shift
+        // parental genotype toward hom.
+
         // Reduce QS for all alleles to account for noise
         for (k=0; k<nqs1; k++)
         {
@@ -1186,6 +1228,7 @@ static void set_trio_QS_noisy(args_t *args, trio_t *trio, double *pqs[3], int nq
             if ( val < 0 ) val = 0;
             pqs[j][k] = phred2log(val);
         }
+#endif
 #if 0
         // The original code, don't like the capping at 255
         // Reduce QS for all alleles to account for noise
@@ -1223,7 +1266,6 @@ static void set_trio_QS_noisy(args_t *args, trio_t *trio, double *pqs[3], int nq
             //      pqs[j][k] = log(1 - exp(phred2log(tmp)));
         }
 #endif
-    }
 }
 static int set_trio_GT(args_t *args, trio_t *trio, int32_t gts[3], int ngts, int ignore_father)
 {
@@ -1632,15 +1674,19 @@ int run(int argc, char **argv)
         {"targets-file",1,0,'T'},
         {"targets-overlap",required_argument,NULL,15},
         {"write-index",optional_argument,NULL,'W'},
+        {"verbosity",required_argument,NULL,'v'},
         {NULL,0,NULL,0}
     };
     int c;
     char *tmp;
     double pn_abs, pn_frac;
-    while ((c = getopt_long(argc, argv, "p:P:o:O:s:i:e:r:R:t:T:m:au:X:nW::",loptions,NULL)) >= 0)
+    while ((c = getopt_long(argc, argv, "p:P:o:O:s:i:e:r:R:t:T:m:au:X:nW::v:",loptions,NULL)) >= 0)
     {
         switch (c)
         {
+            case 'v':
+                if ( apply_verbosity(optarg) < 0 ) error("Could not parse argument: --verbosity %s\n", optarg);
+                break;
             case  1 : args->force_ad = 1; break;
             case  2 : free(args->dnm_score_tag); args->dnm_score_tag = strdup(optarg); break;
             case  3 : free(args->dnm_allele_tag); args->dnm_allele_tag = strdup(optarg); break;
