@@ -1,6 +1,6 @@
 /* The MIT License
 
-   Copyright (c) 2013-2018 Genome Research Ltd.
+   Copyright (c) 2013-2021 Genome Research Ltd.
 
    Author: Petr Danecek <pd3@sanger.ac.uk>
 
@@ -34,6 +34,7 @@
 #include <gsl/gsl_multifit_nlin.h>
 #include <htslib/vcf.h>
 #include <htslib/synced_bcf_reader.h>
+#include <htslib/hts_defs.h>
 #include "bcftools.h"
 #include "peakfit.h"
 
@@ -56,12 +57,13 @@ typedef struct
     char **argv, *output_dir;
     double fit_th, peak_symmetry, cn_penalty, min_peak_size, min_fraction;
     int argc, plot, verbose, regions_is_file, targets_is_file, include_aa, force_cn;
+    int regions_overlap, targets_overlap;
     char *dat_fname, *fname, *regions_list, *targets_list, *sample;
     FILE *dat_fp;
 }
 args_t;
 
-FILE *open_file(char **fname, const char *mode, const char *fmt, ...);
+FILE *open_file(char **fname, const char *mode, const char *fmt, ...) HTS_FORMAT(HTS_PRINTF_FMT, 3, 4);
 
 static void init_dist(args_t *args, dist_t *dist, int verbose)
 {
@@ -75,7 +77,7 @@ static void init_dist(args_t *args, dist_t *dist, int verbose)
     double avg = tmp[0] = dist->yvals[0];
     for (i=1; i<hwin; i++)
     {
-        avg += dist->yvals[2*i-1]; 
+        avg += dist->yvals[2*i-1];
         tmp[i] = avg/(2*i+1);
     }
     avg = 0;
@@ -159,7 +161,7 @@ static void init_dist(args_t *args, dist_t *dist, int verbose)
     dist->ira = n*0.5;
 
     if ( verbose )
-        fprintf(stderr,"%s:\t irr,ira,iaa=%.2f,%.2f,%.2f \t cn=%2d \t ra/rr=%f \t aa/ra=%f \t nra=%d\n", 
+        fprintf(stderr,"%s:\t irr,ira,iaa=%.2f,%.2f,%.2f \t cn=%2d \t ra/rr=%f \t aa/ra=%f \t nra=%d\n",
             dist->chr, dist->xvals[irr],dist->xvals[dist->ira],dist->xvals[iaa],
             dist->copy_number,sra/srr,saa/sra, (int)sra);
 }
@@ -169,11 +171,13 @@ static void init_data(args_t *args)
     bcf_srs_t *files = bcf_sr_init();
     if ( args->regions_list )
     {
+        bcf_sr_set_opt(files,BCF_SR_REGIONS_OVERLAP,args->regions_overlap);
         if ( bcf_sr_set_regions(files, args->regions_list, args->regions_is_file)<0 )
             error("Failed to read the regions: %s\n", args->regions_list);
     }
     if ( args->targets_list )
     {
+        bcf_sr_set_opt(files,BCF_SR_TARGETS_OVERLAP,args->targets_overlap);
         if ( bcf_sr_set_targets(files, args->targets_list, args->targets_is_file, 0)<0 )
             error("Failed to read the targets: %s\n", args->targets_list);
     }
@@ -196,18 +200,14 @@ static void init_data(args_t *args)
     for (i=0; i<args->nbins; i++) args->xvals[i] = 1.0*i/(args->nbins-1);
 
     // collect BAF distributions for all chromosomes
-    int idist = -1, nbaf = 0, nprocessed = 0, ntotal = 0, prev_chr = -1;
+    int idist = -1, nbaf = 0, prev_chr = -1;
     float *baf = NULL;
     while ( bcf_sr_next_line(files) )
     {
-        ntotal++;
-
         bcf1_t *line = bcf_sr_get_line(files,0);
         if ( bcf_get_format_float(hdr,line,"BAF",&baf,&nbaf) != 1 ) continue;
         if ( bcf_float_is_missing(baf[0]) ) continue;
 
-        nprocessed++;
-
         if ( prev_chr==-1 || prev_chr!=line->rid )
         {
             // new chromosome
@@ -536,8 +536,8 @@ static void fit_curves(args_t *args)
             // Use cn_penalty as a tiebreaker. If set to 0.3, cn3_fit must be 30% smaller than cn2_fit.
             if ( cn<0 || cn3_fit < (1-args->cn_penalty) * fit )
             {
-                cn = 2 + cn3_frac; 
-                fit = cn3_fit; 
+                cn = 2 + cn3_frac;
+                fit = cn3_fit;
                 if ( cn2_fail=='*' ) cn2_fail = 'p';
             }
             else cn3_fail = 'p';
@@ -619,24 +619,26 @@ static void usage(args_t *args)
 {
     fprintf(stderr, "\n");
     fprintf(stderr, "About:   Detect number of chromosomal copies from Illumina's B-allele frequency (BAF)\n");
-    fprintf(stderr, "Usage:   bcftools polysomy [OPTIONS] <file.vcf>\n");
+    fprintf(stderr, "Usage:   bcftools polysomy [OPTIONS] FILE.vcf\n");
     fprintf(stderr, "\n");
     fprintf(stderr, "General options:\n");
-    fprintf(stderr, "    -o, --output-dir <path>        \n");
-    fprintf(stderr, "    -r, --regions <region>         restrict to comma-separated list of regions\n");
-    fprintf(stderr, "    -R, --regions-file <file>      restrict to regions listed in a file\n");
-    fprintf(stderr, "    -s, --sample <name>            sample to analyze\n");
-    fprintf(stderr, "    -t, --targets <region>         similar to -r but streams rather than index-jumps\n");
-    fprintf(stderr, "    -T, --targets-file <file>      similar to -R but streams rather than index-jumps\n");
-    fprintf(stderr, "    -v, --verbose                  \n");
+    fprintf(stderr, "    -o, --output-dir PATH          \n");
+    fprintf(stderr, "    -r, --regions REGION           Restrict to comma-separated list of regions\n");
+    fprintf(stderr, "    -R, --regions-file FILE        Restrict to regions listed in a file\n");
+    fprintf(stderr, "        --regions-overlap 0|1|2    Include if POS in the region (0), record overlaps (1), variant overlaps (2) [1]\n");
+    fprintf(stderr, "    -s, --sample NAME              Sample to analyze\n");
+    fprintf(stderr, "    -t, --targets REGION           Similar to -r but streams rather than index-jumps\n");
+    fprintf(stderr, "    -T, --targets-file FILE        Similar to -R but streams rather than index-jumps\n");
+    fprintf(stderr, "        --targets-overlap 0|1|2    Include if POS in the region (0), record overlaps (1), variant overlaps (2) [0]\n");
+    fprintf(stderr, "    -v, --verbosity INT            Verbosity level\n");
     fprintf(stderr, "\n");
     fprintf(stderr, "Algorithm options:\n");
-    fprintf(stderr, "    -b, --peak-size <float>        minimum peak size (0-1, larger is stricter) [0.1]\n");
-    fprintf(stderr, "    -c, --cn-penalty <float>       penalty for increasing CN (0-1, larger is stricter) [0.7]\n");
-    fprintf(stderr, "    -f, --fit-th <float>           goodness of fit threshold (>0, smaller is stricter) [3.3]\n");
-    fprintf(stderr, "    -i, --include-aa               include the AA peak in CN2 and CN3 evaluation\n");
-    fprintf(stderr, "    -m, --min-fraction <float>     minimum distinguishable fraction of aberrant cells [0.1]\n");
-    fprintf(stderr, "    -p, --peak-symmetry <float>    peak symmetry threshold (0-1, larger is stricter) [0.5]\n");
+    fprintf(stderr, "    -b, --peak-size FLOAT          Minimum peak size (0-1, larger is stricter) [0.1]\n");
+    fprintf(stderr, "    -c, --cn-penalty FLOA>         Penalty for increasing CN (0-1, larger is stricter) [0.7]\n");
+    fprintf(stderr, "    -f, --fit-th FLOAT             Goodness of fit threshold (>0, smaller is stricter) [3.3]\n");
+    fprintf(stderr, "    -i, --include-aa               Include the AA peak in CN2 and CN3 evaluation\n");
+    fprintf(stderr, "    -m, --min-fraction FLOAT       Minimum distinguishable fraction of aberrant cells [0.1]\n");
+    fprintf(stderr, "    -p, --peak-symmetry FLOAT      Peak symmetry threshold (0-1, larger is stricter) [0.5]\n");
     fprintf(stderr, "\n");
     exit(1);
 }
@@ -653,6 +655,8 @@ int main_polysomy(int argc, char *argv[])
     args->ra_rr_scaling = 1;
     args->min_fraction = 0.1;
     args->smooth = -3;
+    args->regions_overlap = 1;
+    args->targets_overlap = 0;
 
     static struct option loptions[] =
     {
@@ -663,7 +667,8 @@ int main_polysomy(int argc, char *argv[])
         {"include-aa",0,0,'i'},
         {"peak-size",1,0,'b'},
         {"min-fraction",1,0,'m'},
-        {"verbose",0,0,'v'},
+        {"verbose",optional_argument,0,'v'},
+        {"verbosity",optional_argument,0,'v'},
         {"fit-th",1,0,'f'},
         {"cn-penalty",1,0,'c'},
         {"peak-symmetry",1,0,'p'},
@@ -671,18 +676,28 @@ int main_polysomy(int argc, char *argv[])
         {"sample",1,0,'s'},
         {"targets",1,0,'t'},
         {"targets-file",1,0,'T'},
+        {"targets-overlap",required_argument,NULL,4},
         {"regions",1,0,'r'},
         {"regions-file",1,0,'R'},
+        {"regions-overlap",required_argument,NULL,3},
         {0,0,0,0}
     };
     char *tmp;
     int c;
-    while ((c = getopt_long(argc, argv, "h?o:vt:T:r:R:s:f:p:c:im:b:n:S:",loptions,NULL)) >= 0)
+    while ((c = getopt_long(argc, argv, "h?o:v::t:T:r:R:s:f:p:c:im:b:n:S:",loptions,NULL)) >= 0)
     {
         switch (c)
         {
             case  1 : args->ra_rr_scaling = 0; break;
             case  2 : args->force_cn = atoi(optarg); break;
+            case  3 :
+                args->regions_overlap = parse_overlap_option(optarg);
+                if ( args->regions_overlap < 0 ) error("Could not parse: --regions-overlap %s\n",optarg);
+                break;
+            case  4 :
+                args->targets_overlap = parse_overlap_option(optarg);
+                if ( args->targets_overlap < 0 ) error("Could not parse: --targets-overlap %s\n",optarg);
+                break;
             case 'n': args->nbins = atoi(optarg); break;
             case 'S': args->smooth = atoi(optarg); break;
             case 'i': args->include_aa = 1; break;
@@ -714,7 +729,14 @@ int main_polysomy(int argc, char *argv[])
             case 'r': args->regions_list = optarg; break;
             case 'R': args->regions_list = optarg; args->regions_is_file = 1; break;
             case 'o': args->output_dir = optarg; break;
-            case 'v': args->verbose++; break;
+            case 'v':
+                if ( !optarg ) args->verbose++;
+                else
+                {
+                    args->verbose = strtol(optarg,&tmp,10);
+                    if ( *tmp || args->verbose<0 ) error("Could not parse argument: --verbosity %s\n", optarg);
+                }
+                break;
             default: usage(args); break;
         }
     }
